library(shipunov)

sp <- read.table("../../../data/picramniaceae_sp.txt", h=T, sep="\t", as.is=TRUE)
treesp <- read.table("../../../data/picramniaceae_treesp.txt", h=T, sep="\t", as.is=TRUE)
wanted <- read.table("../../workflow/13_wanted/picramniaceae_wanted.txt", h=T, sep="\t", as.is=TRUE)

out <- treesp$SPECIES.NEW[treesp$TYPE == "outgroup"]
outg <- unique(do.call(rbind, strsplit(out, split=" "))[, 1])

## coverage: stacked barplot per genus -- species vs. wanted
tallg <- table(sp$GENUS)
tallg <- tallg[!names(tallg) %in% outg]

wantg <- do.call(rbind, strsplit(wanted$SPECIES[wanted$HAD.SAMPLE == 0], split=" "))[, 1]
twantg <- table(wantg)
twantg <- twantg[!names(twantg) %in% outg, drop=FALSE] # prevent conversion into vector with NULL dimensions

counts <- merge(Table2df(tallg), Table2df(twantg), by=0, all=TRUE)
row.names(counts) <- counts$Row.names
counts$Row.names <- NULL
names(counts) <- c("sequenced", "wanted")

pdf("data/coverage.pdf", width=8)
barplot(t(counts), xlab="", legend=c("sequenced", "wanted"), args.legend=list(x="top", bty="n"))
dev.off()
