library(shipunov)

dna <- read.table("../../../../data/picramniaceae_dna.txt", sep="\t", h=TRUE, as.is=TRUE)

dna <- dna[dna$SELECT == 1 | is.na(dna$SELECT), ]
dna <- dna[dna$SOURCE %in% "MISU", ]
row.names(dna) <- NULL

split.ids <- c(suppressWarnings(do.call(rbind, strsplit(dna$SEQUENCE.ID, ", "))))
long.dna <- data.frame(ID=split.ids, dna[, c("SPECIES.NEW", "FRAGMENT", "SEQUENCE")])
long.dna <- unique(long.dna)

long.dna <- long.dna[!grepl("^m|^PG-|plantago_", long.dna$ID), ]
long.dna <- long.dna[!long.dna$FRAGMENT %in% c("its"), ]

ff <- function(x) {
if(x == "rbcl") res <- paste(readLines("templates/rbcl.txt"), collapse="\n")
if(x == "trnl") res <- paste(readLines("templates/trnl.txt"), collapse="\n")
res
}

dnas <- split(long.dna[, c("ID", "SEQUENCE", "FRAGMENT")], long.dna$FRAGMENT)
for (s in names(dnas)) {
 subs <- dnas[[s]]
 subs$ID <- apply(subs[, c("ID", "FRAGMENT")], 1, function(.x) paste0(.x[1], "_", .x[2]))
 subs$FEATURES <- ff(s)
 ## 'E!!' variable is sequence length
 for (n in 1:nrow(subs)) subs[n, "FEATURES"] <- gsub("E!!", nchar(subs[n, "SEQUENCE"]), subs[n, "FEATURES"])
 subs <- subs[, c("ID", "FEATURES")]
 subs <- subs[order(subs$ID), c("ID", "FEATURES")]
 subs$ID <- paste("Feature", subs$ID)
 Write.fasta(subs, file=paste0(s, "_features.txt"))
}
