## Picramnia keys and trees

library(shipunov)
library(ape)

kk <- read.table("data/keys.txt", sep="\t", as.is=TRUE)
nk <- Biokey(kk, from="branched", to="newick")
tk <- read.tree(text=Biokey(kk, from="branched", to="newick"))
mk <- MRH(as.hclust(compute.brlen(multi2di(tk, random=FALSE))))
row.names(mk) <- gsub("_", " ", row.names(mk))
row.names(mk) <- gsub(" subsp. .*$", "", row.names(mk))
## merge() multiplies duplicates: merge(data.frame(V1=c("a", "a", "b"), V2=1:3), data.frame(V1=c("a", "b", "c"), V2=4:6), by="V1", all=TRUE)
## and this is good for DNN (classes are larger)

DIR <-  dir("data")
trees <- c("strict_mb", "strict_mp")
for(t in trees) {
assign(t, read.tree(paste0("data/",  DIR[grepl(t, DIR)])))
}

mbc <- MRH(as.hclust(compute.brlen(strict_mb)), method="height", dim=200)
mpc <- MRH(as.hclust(compute.brlen(strict_mp)), method="height", dim=200)
mc <- cbind(mbc, mpc)
colnames(mc) <- paste0("MC", 1:ncol(mc))
mc <- mc[grep("Picramnia", row.names(mc)), ]
row.names(mc) <- gsub("__.*$", "", row.names(mc))
row.names(mc) <- gsub("_", " ", row.names(mc))
row.names(mc) <- gsub("Picramnia aff. glazioviana", "Picramnia glazioviana", row.names(mc))
row.names(mc) <- gsub("Picramnia cf. caracasana", "Picramnia caracasana", row.names(mc))
row.names(mc) <- gsub("Picramnia elliptica Minas Gerais", "Picramnia elliptica", row.names(mc))
row.names(mc) <- gsub("Picramnia oreadica penduliflora", "Picramnia oreadica", row.names(mc))
row.names(mc) <- gsub("Picramnia sp. Nadruz 2552", "Picramnia sellowii", row.names(mc))

missed <- unique(row.names(mk)[!row.names(mk) %in% row.names(mc)])

merged <- merge(mc, mk, by=0, all=TRUE)
cl <- gsub("\\.[1-9]+$", "", merged$Row.names)
merged <- merged[, -1]
cl <- gsub("\\.", " ", cl)
cl <- gsub("andrade limae", "andrade-limae", cl)

MAT <- as.matrix(merged)
row.names(MAT) <- cl

MAT.missed <- MAT[row.names(MAT) %in% missed, ]
MAT.known <- MAT[!row.names(MAT) %in% missed, ]

B <- 1000
PRED <- matrix(nrow=nrow(MAT.missed), ncol=B, dimnames=list(row.names(MAT.missed)))
for (i in 1:B) {
 cat(".")
 MAT.known.b <- MAT.known[sample(nrow(MAT.known), replace=TRUE), ]
 MAT.dist <- dist(rbind(MAT.known.b, MAT.missed), method="manhattan")
 PRED[, i] <- DNN(MAT.dist, factor(c(row.names(MAT.known.b), rep(NA, nrow(MAT.missed)))), k=7)
}
DEV <- vector(mode="list", length=nrow(PRED))
names(DEV) <- row.names(PRED)
for (n in 1:nrow(PRED)) {
 TT <- sort(table(PRED[n, ]), decreasing=TRUE)
 DEV[[n]] <- round(100*(TT[1:3]/B))
 names(DEV[[n]]) <- names(TT)[1:3]
}

placements <- cbind(do.call(rbind, lapply(DEV, names)), do.call(rbind, DEV))
colnames(placements) <- 1:ncol(placements)
placements <- placements[, matrix(colnames(placements), nrow=2, byrow=TRUE)] # interleave columns
write.table(placements, "data/picramniaceae_placements.txt", col.names=FALSE, quote=FALSE, sep="\t")
