library(shipunov)

fragments <- c("its", "rbcl", "trnl") # fragile
sequences <- read.table("../../data/picramniaceae_dna.txt", sep="\t", h=T, as.is=T, quote="")
treesp <- read.table("../../data/picramniaceae_treesp.txt", sep="\t", h=T, as.is=T)
outgroups <- treesp$SPECIES.NEW[treesp$TYPE == "outgroup"]

dna <- structure(vector("list", length(fragments)), names=fragments) # fragile
for (f in fragments) {
dna[[f]] <- Read.fasta(paste0("33_alignments_trimmed_gapcoded/", f, ".fasta"))
sp.id <- do.call(rbind, strsplit(dna[[f]]$name, "__"))
dna[[f]] <- data.frame(SP=sp.id[, 1], ID=sp.id[, 2], SEQ=dna[[f]][, 2], stringsAsFactors=FALSE)
dna[[f]] <- dna[[f]][order(nchar(gsub("-", "", dna[[f]]$SEQ)), decreasing=TRUE), ]
dna[[f]] <- dna[[f]][order(dna[[f]]$SP), ]
}

## join all sp/ids from list
sp.ids <- do.call(rbind, c(lapply(dna, "[", 1:2), make.row.names=F))
tmp1 <- suppressWarnings(do.call(rbind, strsplit(sp.ids$ID, ", ")))
tmp2 <- unique(sort(paste(c(tmp1), sp.ids$SP, sep="__")))
ids.sp <- do.call(rbind, strsplit(tmp2, "__"))

## ===
## ===
## ===

## STEP 1. STRICT: its, rbcl and trnl merged on "our" ids

## rows
tmp1 <- sequences[sequences$SOURCE == "MISU", "SEQUENCE.ID"]
tmp2 <- suppressWarnings(do.call(rbind, strsplit(tmp1, ", ")))
ourids.all <- unique(sort(tmp2))
ourids <- ids.sp[ids.sp[, 1] %in% ourids.all, ]

## make
strict <- data.frame(ourids, stringsAsFactors=F)
names(strict) <- c("ID", "SP")

## outgroups first, other sorted
tmp1 <- strict[strict$SP %in% outgroups, ]
tmp2 <- strict[!strict$SP %in% outgroups, ]
tmp2 <- tmp2[order(tmp2$SP), ]
strict <- rbind(tmp1, tmp2)

for (n in fragments) {
 strict[[n]] <- NA
 for (i in 1:nrow(strict)) {
  ii <- dna[[n]]$SEQ[grepl(strict$ID[i], dna[[n]]$ID)]
  strict[[n]][i] <- ifelse(length(ii) == 0, NA, ii)
 }
}

## ===
## ===
## ===

## STEP 2. SEMI-STRICT (the main matrix):
## add new species which were not in "our" ids;
## fill species sequences into all NA gaps and keep that id

semistrict <- strict

## rows
ssp <- sort(unique(semistrict$SP))
allsp <- sort(unique(sp.ids$SP))
addsp <- allsp[!allsp %in% ssp]
if (length(addsp) > 0) {
semistrict[(nrow(semistrict) + 1) : (nrow(semistrict) + length(addsp)), ] <- NA
semistrict[nrow(semistrict) : (nrow(semistrict) - (length(addsp) - 1)), "SP"] <- addsp
semistrict[nrow(semistrict) : (nrow(semistrict) - (length(addsp) - 1)), "ID"] <- NA
}

## outgroups first, other sorted
tmp1 <- semistrict[semistrict$SP %in% outgroups, ]
tmp2 <- semistrict[!semistrict$SP %in% outgroups, ]
tmp2 <- tmp2[order(tmp2$SP), ]
semistrict <- rbind(tmp1, tmp2)

## columns
names.tmp3 <- c(paste0(fragments, ".ID"))
tmp3 <- structure(data.frame(matrix(NA, nrow=nrow(semistrict), ncol=length(names.tmp3))), names=names.tmp3)
semistrict <- cbind(semistrict, tmp3)
semistrict <- semistrict[, c("SP", "ID", paste0(fragments, ".ID"), fragments)]

## main
for(n in fragments) {
 d <- dna[[n]]
 f.id <- paste0(n, ".ID")
 for (i in 1:nrow(semistrict)) {
  if (is.na(semistrict[i, n])) {
  dd <- d$SEQ[grepl(semistrict$SP[i], d$SP)]
  ii <- d$ID[grepl(semistrict$SP[i], d$SP)]
  if (length(dd) > 0 & length(ii) > 0) {
   semistrict[i, n] <- dd[1]
   semistrict[i, f.id] <- ii[1]
   } else {
   semistrict[i, f.id] <- "-"
   }
 }}
}

## ===
## ===
## ===

## STEP 3. Write files

## ===
## semistrict
## ===

## make joint ids
semistrict.f.ids <- names.tmp3
ids.tmp0 <- semistrict[, c("SP", "ID", semistrict.f.ids)]
ids.tmp1 <- apply(ids.tmp0[, c("SP", "ID")], 1, function(.x) paste(.x, collapse="__"))
for (i in semistrict.f.ids) ids.tmp0[, i] <- ifelse(is.na(ids.tmp0[, i]), "", paste(sub(".ID", "", i), ids.tmp0[, i]))
ids.tmp2 <- apply(ids.tmp0[, semistrict.f.ids], 1, function(.x) paste(.x, collapse=" "))
ids.tmp2 <- gsub("(^ +)|( +$)", "", ids.tmp2)
ids.tmp2 <- gsub("( +)", " ", ids.tmp2)
ids.tmp <- paste(ids.tmp1, ids.tmp2)
ids.tmp <- gsub(" +", "_", ids.tmp)
ids.tmp <- gsub(",", "", ids.tmp)
ids.tmp <- gsub("_$", "", ids.tmp)
ids.tmp <- gsub("_NA_", "_", ids.tmp)

## duplicates
semistrict.f <- fragments
groups <- as.numeric(as.factor(apply(semistrict[, c("SP", semistrict.f)], 1, paste, collapse="")))
comb.ids <-  aggregate(ids.tmp, by=list(groups), paste, collapse=" + ")
semistrict <- semistrict[!duplicated(groups), ] # remove duplicates in main table
ids <- ids.tmp[!duplicated(groups)] # remove duplicates in joint ids
comb.ids <- comb.ids[groups[!duplicated(groups)], 2] # the same order as in deduplicated data
## this table is a "filter" for tree labels
write.table(data.frame(ids, comb.ids), file="40_concatenated/semistrict_comb_ids.txt", sep="\t", quote=F, row.names=F)

## semistrict statistics
semistrict.m <- data.frame(semistrict[, 1:2], apply(semistrict[, semistrict.f], 2, function(.y) ifelse(is.na(.y), 0, 1)))
semistrict.m$COUNT <- rowSums(semistrict.m[, semistrict.f])
## what missed after concatenation
semistrict.m[order(semistrict.m$COUNT, decreasing=T), ]
## how many ids in each missing state
table(apply(semistrict[, semistrict.f], 1, function(.y) sum(!is.na(.y))))
## how many ids missed in each fragment
apply(semistrict[, semistrict.f], 2, function(.x) paste0(sum(is.na(.x)), "/", length(.x)))
## how many sequences our and total
all.ids <- unlist(semistrict[, c("ID", paste0(semistrict.f, ".ID"))])
## our
sum(all.ids %in% ourids.all & !is.na(all.ids) & all.ids != "-")
## total
sum(!is.na(all.ids) & all.ids != "-")
## number of potentially informative characters and total number of characters per fragment
for (f in semistrict.f) {
cat(f)
tmp <- semistrict[, f]
tmp <- tmp[!is.na(tmp)]
tmp.table <- do.call(rbind, strsplit(tmp, split=""))
tmp.table[tmp.table == "-" | tmp.table == "N"] <- NA
cat("\tall chars ", ncol(tmp.table))
tmp.itic <- apply(tmp.table, 2, Is.tax.inform.char)
cat("\tpotentially taxonomically informative chars ", sum(tmp.itic), "\n")
}

## write original semistrict matrix (for imputation)
write.table(semistrict, file=paste0("40_concatenated/", "semistrict_orig.txt"), sep="\t", row.names=F, quote=F)

## replace NA's with N's and write resulted FASTA
for (i in semistrict.f) semistrict[, i] <- ifelse(is.na(semistrict[, i]), paste(rep("N", max(nchar(semistrict[, i]), na.rm=T)), collapse=""), semistrict[, i])
seq <- apply(semistrict[, semistrict.f], 1, function(.x) paste(.x, collapse=""))
semistrict.new <- data.frame(ID=ids, SEQ=seq, stringsAsFactors=F)
Write.fasta(semistrict.new, file=paste0("40_concatenated/", "semistrict.fasta"))

## ===
## strict
## ===

data.f <- c("its", "rbcl", "trnl")
data <- strict[, c("SP", "ID", data.f)]

## make joint ids
ids.tmp0 <- data[, c("SP", "ID")]
ids.tmp <- apply(ids.tmp0[, c("SP", "ID")], 1, function(.x) paste(.x, collapse="__"))
ids.tmp <- gsub(" +", "_", ids.tmp)
ids.tmp <- gsub(",", "", ids.tmp)
ids.tmp <- gsub("_$", "", ids.tmp)
ids.tmp <- gsub("_NA_", "_", ids.tmp)

## duplicates
groups <- as.numeric(as.factor(apply(data[, c("SP", data.f)], 1, paste, collapse="")))
comb.ids <-  aggregate(ids.tmp, by=list(groups), paste, collapse=" + ")
data <- data[!duplicated(groups), ] # remove duplicates in main table
ids <- ids.tmp[!duplicated(groups)] # remove duplicates in joint ids
comb.ids <- comb.ids[groups[!duplicated(groups)], 2] # the same order as in deduplicated data
## this table is a "filter" for tree labels
write.table(data.frame(ids, comb.ids), file="40_concatenated/strict_comb_ids.txt", sep="\t", quote=F, row.names=F)

## strict statistics
data.m <- data.frame(data[, c("SP", "ID")], apply(data[, data.f], 2, function(.y) ifelse(is.na(.y), 0, 1)))
data.m$COUNT <- rowSums(data.m[, data.f])
## what missed after concatenation
data.m[order(data.m$COUNT, decreasing=T), ]
## how many ids in each missing state
table(apply(data[, data.f], 1, function(.y) sum(!is.na(.y))))
## how many ids missed in each fragment
apply(data[, data.f], 2, function(.x) paste0(sum(is.na(.x)), "/", length(.x)))
## how many sequences our and total
all.ids <- unlist(data[, "ID"])
## our
sum(all.ids %in% ourids.all & !is.na(all.ids) & all.ids != "-")
## total
sum(!is.na(all.ids) & all.ids != "-")
## number of potentially informative characters and total number of characters per fragment
for (f in data.f) {
cat(f)
tmp <- data[, f]
tmp <- tmp[!is.na(tmp)]
tmp.table <- do.call(rbind, strsplit(tmp, split=""))
tmp.table[tmp.table == "-" | tmp.table == "N"] <- NA
cat("\tall chars ", ncol(tmp.table))
tmp.itic <- apply(tmp.table, 2, Is.tax.inform.char)
cat("\tpotentially taxonomically informative chars ", sum(tmp.itic), "\n")
}

## replace NA's with N's and write resulted FASTA
for (i in data.f) data[, i] <- ifelse(is.na(data[, i]), paste(rep("N", max(nchar(data[, i]), na.rm=T)), collapse=""), data[, i])
seq <- apply(data[, data.f], 1, function(.x) paste(.x, collapse=""))
data.new <- data.frame(ID=ids, SEQ=seq, stringsAsFactors=F)
Write.fasta(data.new, file=paste0("40_concatenated/", "strict.fasta"))
